* PRISMOD MODEL - dopris0_master.do *
* Libor Dusek, Jan Vavra, Paolo Buonanno
** more info in dopris0_readme.do

**** SOME AD-HOC POLICIES FOR SPECIFIC STUDIES
clear
**** switches ********
scalar s_sent20 = 0
scalar s_pe_a320 = 0
scalar s_tejc = 1
**********************
global pcent "100" 

if s_sent20 == 1 {
**** sentb20
**** 1. INCREASE SELECTED SENTENCES BY 20% FOR
* oca3: theft/property
* oca5: driving
* oca8: family
* starting 2017, follow-up with conversions in 2018

* 1.1 Start with the base, save new directory, new file
clear
set more off
capture log close
global pcent "100"

capture mkdir simulations_$pcent/sentb20
use simulations_$pcent/base/base_paramavg_$pcent
save simulations_$pcent/sentb20/sentb20_paramavg_$pcent, replace

*** 1.2. Collect the average value of relevant sentences in 2016, and 
*** replace 2017 and after with those values
* list the parameters and the corresponding starting years
local parlist "s_ls s_hs s_a1 s_a2 s_a3"
local startyear "2017 2017 2018 2018 2018 2018"
* list the offense categories to which the change applies
local ocalist "3 5 8"
local npars: word count `parlist'


*loop over oca's and parametrs, increase them by 30% after the respective startyear

forvalues i = 1/`npars' {
	local par: word `i' of `parlist'
	local sy: word `i' of `startyear'
	foreach j of local ocalist {
		bysort oca: egen x`par' = mean(`par') if q>=tq(2016q1) & q<=tq(2016q4) & oca==`j'
		replace x`par' = 1.20*x`par'
		bysort oca: egen xe`par' = max(x`par') if oca==`j'
		replace `par' = xe`par' if q>=tq(`sy'q1) & oca==`j'
		drop x`par' xe`par'
	}
}

save simulations_$pcent/sentb20/sentb20_paramavg_$pcent, replace

*** 1.3 Conduct the standard model simulation and comparisons
quietly do dopris3_modelavg
	pminflowsimul sentb20
quietly do dopris4_descriptive
	modelcompare sentb20 base
}


if s_pe_a320 == 1 {	
**************************************
**** pe_a320
**************************************
**** 1. INCREASE THE FRACTION RELEASED EARLY BY 20% FOR ALL

* 1.1 Start with the base, save new directory, new file
clear
set more off
capture log close
global pcent "100"

capture mkdir simulations_$pcent/pe_a320
use simulations_$pcent/base/base_paramavg_$pcent
save simulations_$pcent/pe_a320/pe_a320_paramavg_$pcent, replace

*** 1.2. Collect the average value of relevant sentences in 2016, and 
*** replace 2017 and after with those values
* list the parameters and the corresponding starting years
local parlist "pe_a3"
local startyear "2017"
* list the offense categories to which the change applies
local ocalist "1 2 3 4 5 6 7 8 9"
local npars: word count `parlist'


*loop over oca's and parametrs, increase them by 30% after the respective startyear

forvalues i = 1/`npars' {
	local par: word `i' of `parlist'
	local sy: word `i' of `startyear'
	foreach j of local ocalist {
		bysort oca: egen x`par' = mean(`par') if q>=tq(2016q1) & q<=tq(2016q4) & oca==`j'
		replace x`par' = 1.20*x`par'
		bysort oca: egen xe`par' = max(x`par') if oca==`j'
		replace `par' = xe`par' if q>=tq(`sy'q1) & oca==`j'
		drop x`par' xe`par'
	}
}

save simulations_$pcent/pe_a320/pe_a320_paramavg_$pcent, replace

** 1.3 Conduct the standard model simulation and comparisons
quietly do dopris3_modelavg
	pminflowsimul pe_a320
quietly do dopris4_descriptive
	modelcompare tejc base
}

if s_tejc == 1 {	
**************************************
**** Tejc - sentence increase for serious offences
**************************************
** sentence change of serious offences
** aims to match punishment for violent and economics offences
** assumed change for murderers, para 142 col 2 and 3 gets life sentence and they go to HS prisons

clear
set more off
capture log close
global pcent "100"

capture mkdir simulations_$pcent/tejc
use simulations_$pcent/base/base_paramavg_$pcent
save simulations_$pcent/tejc/tejc_paramavg_$pcent, replace

/* estimated scale factors from microdata, inputed 25y for para 142, col2 and col3	
	 +-----------------------------------------------------------------------------------------------------------------------+
     | rok_moc      p4_hs      s_all       s_ls       s_hs     a_s_ls     a_s_hs    a_p4_hs   scale~ls   sca~s_hs   sca~4_hs |
     |-----------------------------------------------------------------------------------------------------------------------|
     |    2011   .5816327   62.16327   39.56098   78.42105   41.54861   58.83858   .6093732   .9521613   1.332817    .954477 |
     |    2012   .6774194   65.93549   36.33333   80.03175   41.54861   58.83858   .6093732   .8744777   1.360192   1.111666 |
     |    2013       .625   62.15384   38.61538   76.27692   41.54861   58.83858   .6093732   .9294025   1.296376   1.025644 |
     |    2014   .7872341   73.17376       34.7   83.57207   41.54861   58.83858   .6093732   .8351663   1.420362   1.291875 |
     +-----------------------------------------------------------------------------------------------------------------------+	
*/
use simulations_$pcent/tejc/tejc_paramavg_$pcent, replace
* use scale factors from 2014 from 1.1. 2017
replace s_ls=s_ls*0.835 if q>=tq(2017q1) & oca==1
replace s_hs=s_hs*1.42 if q>=tq(2017q1) & oca==1
replace p4_hs=p4_hs*1.29 if q>=tq(2017q1) & oca==1
replace p4_hs=1 if p4_hs>1

save simulations_$pcent/tejc/tejc_paramavg_$pcent, replace
set more off
set trace off                
** run the model
cap quietly  do dopris3_modelavg
* set simulation horizon to 2055q4 
global prismod_end = tq(2055q4)
pminflowsimul tejc
quietly do dopris4_descriptive
modelcompare tejc base	
		 
	 
** make some specific plots for this time horizon
use simulations_100\tejc\tejc_basecompare_100.dta
* total
line base_totstock totstock q if oca==3 & q>tq(2015q1) & q<tq(2055q1), tline(2017q1) title("Total stock")
graph export simulations_100\tejc\1_total.png, width(1024) replace
* change
g diff1=totstock - base_totstock
line diff1 q if oca==3 & q>tq(2015q1) & q<tq(2055q1), tline(2017q1) title("Diff between Tejc and averages scenario")
graph export simulations_100\tejc\2_totaldiff.png, width(1024) replace
* by oca
line base_stock stock q if q>tq(2015q1) & q<tq(2055q1), by(oca, yrescale  title("Total stock by oca") ) tline(2017q1)
graph export simulations_100\tejc\3_totaloca.png, width(1024) replace
* change
g diff2=stock-base_stock
line diff2 q if q>tq(2015q1) & q<tq(2055q1), by(oca, yrescale title("Diff between Tejc and averages scenario, by oca") )  tline(2017q1)
graph export simulations_100\tejc\3_totaloca.png, width(1024) replace

* total ls/hs
line base_totstock_ls totstock_ls q if oca==3 & q>tq(2015q1) & q<tq(2055q1), tline(2017q1) title("Total stock LS")
graph export simulations_100\tejc\4_totalls.png, width(1024) replace

line base_totstock_hs totstock_hs q if oca==3 & q>tq(2015q1) & q<tq(2055q1), tline(2017q1) title("Total stock HS")
graph export simulations_100\tejc\5_totalhs.png, width(1024) replace

* change ls/hs
g diff3=totstock_ls - base_totstock_ls
line diff3 q if oca==3 & q>tq(2015q1) & q<tq(2055q1), tline(2017q1) title("Diff between Tejc and averages scenario, LS")
graph export simulations_100\tejc\6_diffls.png, width(1024) replace

g diff4=totstock_hs - base_totstock_hs
line diff4 q if oca==3 & q>tq(2015q1) & q<tq(2055q1), tline(2017q1) title("Diff between Tejc and averages scenario, HS")
graph export simulations_100\tejc\7_diffhs.png, width(1024) replace

* by oca ls/hs
line base_stock_ls stock_ls q if q>tq(2015q1) & q<tq(2055q1), by(oca, yrescale  title("Total stock by oca LS") ) tline(2017q1)
graph export simulations_100\tejc\8_totalocals.png, width(1024) replace

line base_stock_hs stock_hs q if q>tq(2015q1) & q<tq(2055q1), by(oca, yrescale  title("Total stock by oca HS") ) tline(2017q1)
graph export simulations_100\tejc\9_totalocahs.png, width(1024) replace

* change by oca ls/hs
g diff5=stock_ls-base_stock_ls
line diff5 q if q>tq(2015q1) & q<tq(2055q1), by(oca, yrescale title("Diff between Tejc and averages scenario, LS") ) tline(2017q1)
graph export simulations_100\tejc\10_lsdiffoca.png, width(1024) replace

g diff6=stock_hs-base_stock_hs
line diff6 q if q>tq(2015q1) & q<tq(2055q1), by(oca, yrescale title("Diff between Tejc and averages scenario, HS") )  tline(2017q1)
graph export simulations_100\tejc\11_hsdiffoca.png, width(1024) replace

}

