* PRISMOD MODEL - dopris4_descriptive.do *
* Libor Dusek, Jan Vavra, Paolo Buonanno
** more info in dopris0_readme.do

** produces plots for the model
*** contains three programs:
**** 1. paraplot `p' - plot parameters into `p'/para directory  
**** 2. modelfit `p' - merge model with real data, produce plots and fit statistics to `p'/modelfit
**** 3. modelcompare `p1' `p2' - compares `p1' with `p2'

global plot_n=0
global pcent "100" 

**** 1. PARAPLOT *********
quietly {

capture program drop paramgraph
program define paramgraph
	args p x t title
	global plot_n = $plot_n +1
	line /* twoway connect */ `x' `t' if q>=tq(2000q1) & q<=tq(2024q4), by(oca, yrescale title("`title'")) msize(0)
	graph export "simulations_$pcent\\`p'\\para\\{$plot_n}_`p'_paramg_`x'.png", width(2000) replace
	noisily display $plot_n
end

set more off
capture program drop paraplot
program define paraplot
	args p	
	
* create folders or remove old ones
	!rmdir simulations_$pcent\\`p'\para /s /q
	capture mkdir simulations_$pcent/`p'\para
	
*** 1. Model parameters - by offense category
use simulations_$pcent/`p'/`p'_wmodelavg_$pcent
capture drop if oca==10
* do it with a simple program

paramgraph `p' c1 q "Number of tr. rizeni started"
paramgraph `p' z1 q "Average tr. rizeni duration"
paramgraph `p' p1 q "Probability of tr. stihani"
paramgraph `p' p12 q "TECH - balance between police cases and prosecutors data"
paramgraph `p' z2 q "Average case duration - prosecutors (quarters)"
paramgraph `p' p2 q "Probability of charges"
paramgraph `p' p23 q "TECH - balancing between prosec. and court data"
paramgraph `p' z3 q "Average case duration - courts (quarters)"
paramgraph `p' p3 q "Probability of conviction"
paramgraph `p' p4 q "Probability of prison sentence"
paramgraph `p' p_a1 q "Probability of parole sentence"
paramgraph `p' p_a2 q "Probability of public services sentence"
paramgraph `p' p5 q "Probability of nonentered sentence"
paramgraph `p' p45 q "TECH - balancing between court and prison data"
paramgraph `p' vr_a1 q "Rate of entry from parole"
paramgraph `p' vr_a2 q "Rate of entry from community service"
paramgraph `p' vr_a3 q "Rate of entry from prison parole"
paramgraph `p' la1  q "Length of probationary period (quarters)"
paramgraph `p' la2  q "Length of probationary period for the community service (quarters)"
paramgraph `p' la3  q "Length of probationary period for the prison paroled (quarters)"
paramgraph `p' lc q "Average length of custody (quarters)"
paramgraph `p' s_ls  q "Sentence length, court data, regular LS prisoners"
paramgraph `p' s_hs  q "Sentence length, court data, regular HS prisoners"
paramgraph `p' p4_hs  q "Probability of sentencing to HS prison"
paramgraph `p' s_a1 q "Sentence length - parole violations (quarters)"
paramgraph `p' s_a2 q "Sentence length - community service violations (quarters)"
paramgraph `p' s_a3 q "Sentence length - prison parole violations (quarters)"
paramgraph `p' pe_a3 q "Probability of exit to parole"
paramgraph `p' pe_o q "Probability of exit to next sentence"
paramgraph `p' pe_hsls  q "Probability of exit to LS prison, from HS stock (yearbook)"
paramgraph `p' p_z4  q "Fraction of prison stock entered in 1st quarter"
clear

end
}
**** 2. MODELFIT *********
quietly {
capture program drop rmse
program define rmse, rclass byable(recall)
	args timeseries1 timeseries2 timevar timemin timemax 
	syntax anything [if] 
	quietly {
	set more off
	marksample marked
	* generate diff 
	g tmp_timediff=(`timeseries1'-`timeseries2') if `timevar'<=tq(`timemax') & `timevar'>=tq(`timemin') & `marked'
	sum tmp_timediff if `timevar'<=tq(`timemax') & `timevar'>=tq(`timemin') & `marked'
	scalar Av_diff=r(mean)
	g abs_timediff=abs(tmp_timediff) 
	sum abs_timediff if `timevar'<=tq(`timemax') & `timevar'>=tq(`timemin') & `marked'
	scalar Abs_diff=r(mean)
	* square it
	replace tmp_timediff=tmp_timediff^2 if `timevar'<=tq(`timemax') & `timevar'>=tq(`timemin') & `marked'
	* calculate RMSE as scalar
	sum tmp_timediff if `timevar'<=tq(`timemax') & `timevar'>=tq(`timemin') & `marked'
	scalar RMSE = (r(sum)/r(N))^(1/2)
	* calculate normalized RMSE
	sum `timeseries1' if `timevar'<=tq(`timemax') & `timevar'>=tq(`timemin') & `marked'
	scalar nRMSE=RMSE/(r(max)-r(min))
	* drop temp
	drop tmp_timediff 
	drop abs_timediff
	* display RMSE
	}
	display "Average difference: " Av_diff
	display "Average absolute difference: " Abs_diff
	display "RMSE: " RMSE
	display "Normalized RMSE: " nRMSE
end


capture program drop modelfit
program define modelfit
	args p
	set more off
	set trace off
	* create folders or remove old ones
	!rmdir simulations_$pcent\\`p'\\modelfit /s /q
	capture mkdir simulations_$pcent/`p'\modelfit\	
	use simulations_$pcent/`p'/`p'_wmodelavg_$pcent
	* merge real stocks of prisoners in	
	* For comparison with reality, add the actual stock from the yearbook
	merge m:1 q using simulations_100/base/prison_stock_q
	label var totaystock "actual stock of prisoners (yearbook)"
	g totaynetflow=totaystock[_n-1]-totaystock
	label var totaynetflow "actual netflow of prisoners (yearbook)"
	drop _merge
	sort oca q
	
	save simulations_$pcent/`p'/`p'_wmodelavgfit_$pcent, replace
	
	* PLOTS
	* total series
	line totstock totaystock q if q<=tq(2024q4) & q>=tq(2000q1) & oca==3, connect(d d) title("Total simulated stock and yearbook stock")
	graph export simulations_$pcent/`p'/modelfit/totalfit_1.png, width(2000) replace

	* total by oca
	line stock adstock q if q<=tq(2024q4) & q>=tq(2000q1), by(oca, yrescale title("Total simulated stock and real stock by oca"))
	graph export simulations_$pcent/`p'/modelfit/totalfit_2.png, width(2000) replace

	* total, regular prisoners (ls+lshs+hs)
	line totstock_rp totadstock_rp q if q<=tq(2024q4) & q>=tq(2000q1) & oca==3, connect(d d) title("Total simulated and real stock, regular prisoners")	
	graph export simulations_$pcent/`p'/modelfit/totalfit_3.png, width(2000) replace
	
	** low security, standart
	line totstock_ls totaystock_ls q if q<=tq(2024q4) & q>=tq(2000q1) & oca==3, connect(d d) title("Total simulated and real stock, low security regular")
	graph export simulations_$pcent/`p'/modelfit/totalfit_4.png, width(2000) replace
	
	**** transfered HS to LS
	line totstock_lshs q if q<=tq(2024q4) & q>=tq(2000q1) & oca==3, connect(d d) title("Total simulated stock, hs in ls prisons")
	graph export simulations_$pcent/`p'/modelfit/totalfit_5.png, width(2000) replace	
	
	** HS
	line totstock_hs totaystock_hs q if q<=tq(2024q4) & q>=tq(2000q1) & oca==3, connect(d d) title("Total simulated and real stock, high security regular")
	graph export simulations_$pcent/`p'/modelfit/totalfit_6.png, width(2000) replace
	
	** alternative
	line totstock_a1 totadstock_a1 q if q<=tq(2024q4) & q>=tq(2000q1) & oca==3, connect(d d) title("Total simulated and real stock, parole violators")
	graph export simulations_$pcent/`p'/modelfit/totalfit_7.png, width(2000) replace	
	
	line totstock_a2 totadstock_a2 q if q<=tq(2024q4) & q>=tq(2000q1) & oca==3, connect(d d) title("Total simulated and real stock, pub. service violators")
	graph export simulations_$pcent/`p'/modelfit/totalfit_8.png, width(2000) replace
	
	line totstock_a3 totadstock_a3 q if q<=tq(2024q4) & q>=tq(2000q1) & oca==3, connect(d d) title("Total simulated and real stock, prison parole violators")
	graph export simulations_$pcent/`p'/modelfit/totalfit_9.png, width(2000) replace
	
	
	** by oca
	line stock_rp adstock_rp q if q<=tq(2024q4) & q>=tq(2000q1), by(oca, yrescale title("Simulated and real stock, regular prisoners by oca"))
	graph export simulations_$pcent/`p'/modelfit/totalfit_10.png, width(2000) replace
	
	line stock_ls q if q<=tq(2024q4) & q>=tq(2000q1), by(oca, yrescale title("Simulated stock, low security regular prisoners by oca"))
	graph export simulations_$pcent/`p'/modelfit/totalfit_11.png, width(2000) replace
	
	line stock_lshs q if q<=tq(2024q4) & q>=tq(2000q1), by(oca, yrescale title("Simulated stock, hs in ls prisons by oca"))
	graph export simulations_$pcent/`p'/modelfit/totalfit_12.png, width(2000) replace
	
	line stock_hs q if q<=tq(2024q4) & q>=tq(2000q1), by(oca, yrescale title("Simulated and real stock, high security regular prisoners by oca"))
	graph export simulations_$pcent/`p'/modelfit/totalfit_13.png, width(2000) replace
		
	line stock_a1 adstock_a1 q if q<=tq(2024q4) & q>=tq(2000q1), by(oca, yrescale title("Simulated and real stock, parole violators by oca"))
	graph export simulations_$pcent/`p'/modelfit/totalfit_14.png, width(2000) replace
	
	line stock_a2 adstock_a2 q if q<=tq(2024q4) & q>=tq(2000q1), by(oca, yrescale title("Simulated and real stock, pub. service violators by oca"))
	graph export simulations_$pcent/`p'/modelfit/totalfit_15.png, width(2000) replace
	
	line stock_a3 adstock_a3 q if q<=tq(2024q4) & q>=tq(2000q1), by(oca, yrescale title("Simulated and real stock, prison parole violators by oca"))
	graph export simulations_$pcent/`p'/modelfit/totalfit_16.png, width(2000) replace
	
	* OUTFILE WITH STATISTICS
	log using simulations_$pcent/`p'/modelfit/statfit, replace
	display ""
	display "Total stock 2002-2014"
	rmse totstock totadstock q 2002q1 2014q4
	display ""
	display "Total stock 2004-2012" as result
	rmse totstock totadstock q 2004q1 2012q4
	display ""
	display "Total stock by oca 2002-2014" as result
	by oca: rmse stock adstock q 2002q1 2014q4
	display "Total stock by oca 2004-2012" as result
	by oca: rmse stock adstock q 2004q1 2012q4
	log close
	
	* OUTFILE TABLE
	** by quarter
	g diff_totstock=-totadstock+totstock
	g diff_stock=-adstock+stock
	log using simulations_$pcent/`p'/modelfit/tablefit, replace
	display "Total stock"
	rmse totstock totadstock q 2001q1 2014q1
	display ""
	list q totstock  totadstock diff_totstock if oca==3, noobs nocompress table ab(33) separator(4)
	display "Total stock by oca"
	display ""
	forval ocas= 1/9 { 
	local label: label ocalabel `ocas' 
	di "For `label' oca: "
	rmse stock adstock q 2001q1 2014q1 if oca==`ocas'
	list q stock adstock  diff_stock if oca==`ocas', noobs nocompress table ab(33) separator(4)
	display ""
	}
	log close
	drop diff*
	clear
	*save simulations_$pcent/`p'/`p'_wmodelavgfit_$pcent, replace
	
end
}
**** 3. MODELCOMPARE *****
* simple program to plot differences between assumptions
* makes plots and new datafile in modelfile1 folder
capture program drop modelcompare
program define modelcompare
	* modelcompare alin2012 base
	* modelcomapre what with
	args modelfile1 modelfile2 
	* 1. Merge files together, make new dataset
	use simulations_$pcent/`modelfile2'/`modelfile2'_wmodelavg_$pcent 
	save simulations_$pcent/`modelfile1'/`modelfile1'_wmodelcompare_$pcent, replace
	* rename variables
	ds oca q, not
	foreach var in `r(varlist)' {
		rename `var' `modelfile2'_`var' 	
		}	
	save simulations_$pcent/`modelfile1'/`modelfile1'_wmodelcompare_$pcent, replace
	sort oca q
	merge 1:1 oca q using simulations_$pcent/`modelfile1'/`modelfile1'_wmodelavg_$pcent
	drop _merge
	save simulations_$pcent/`modelfile1'/`modelfile1'_`modelfile2'compare_$pcent, replace
* 2. Make some plots
* Store them in modelfile1/comp
capture mkdir simulations_$pcent/`modelfile1'/comp

** plot all paramethers
 foreach var in c1 p1 z1 z2 p2 z3 p3 p4 p_a1 p_a2 s_hs s_ls p4_hs lc la1 la2 la3 p5 pe_a3 pe_o pe_hsls p_z4 s_a1 s_a2 s_a3 vr_a1 vr_a2 vr_a3 {
 local vtext : variable label `var' 
	if `"`vtext'"' == "" local vtext "`var'" 
 line `var' `modelfile2'_`var' q if q>=tq(2001q1) & q<=tq(2024q2), by(oca, yrescale title(`"Parameter comparison"') subtitle(`"`var' - `vtext'"')) legend(label(1 `"`modelfile1' `var'"') label(2 `"`modelfile2' `var'"')) 
 graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_para_`var'.png, width(2000) replace
 }

	* compare total stocks
	
		line `modelfile2'_totstock totstock q if oca==3 & q>=tq(2001q1) & q<=tq(2024q2), title(`"Comparison - total stock"') legend(label(1 `"`modelfile2' total stock"') label(2 `"`modelfile1' total stock"'))
		graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_1tot.png, width(2000) replace
		
		line `modelfile2'_totstock_rp totstock_rp q if oca==3 & q>=tq(2001q1) & q<=tq(2024q2), title(`"Comparison - total regular prisoners stock "') legend(label(1 `"`modelfile2' rp stock"') label(2 `"`modelfile1' rp stock"'))
		graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_2rp.png, width(2000) replace
		
		line `modelfile2'_totstock_hs totstock_hs q if oca==3 & q>=tq(2001q1) & q<=tq(2024q2), title(`"Comparison - total high-security prisoners"') legend(label(1 `"`modelfile2' hs stock"') label(2 `"`modelfile1' hs stock"'))
		graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_2hs.png, width(2000) replace
		
		line `modelfile2'_totstock_ls totstock_ls q if oca==3 & q>=tq(2001q1) & q<=tq(2024q2), title(`"Comparison - total low-security prisoners"') legend(label(1 `"`modelfile2' ls stock"') label(2 `"`modelfile1' ls stock"'))
		graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_2ls.png, width(2000) replace

		line `modelfile2'_totstock_a1 totstock_a1 q if oca==3 & q>=tq(2001q1) & q<=tq(2024q2), title(`"Comparison - total parole violators stock "') legend(label(1 `"`modelfile2' a1 stock"') label(2 `"`modelfile1' a1 stock"'))
		graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_3a1.png, width(2000) replace

		line `modelfile2'_totstock_a2 totstock_a2 q if oca==3 & q>=tq(2001q1) & q<=tq(2024q2), title(`"Comparison - total com. serv. violators stock "') legend(label(1 `"`modelfile2' a2 stock"') label(2 `"`modelfile1' a2 stock"'))
		graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_4a2.png, width(2000) replace
		
		line `modelfile2'_totstock_a3 totstock_a3 q if oca==3 & q>=tq(2001q1) & q<=tq(2024q2), title(`"Comparison - total prison parole stock "') legend(label(1 `"`modelfile2' a3 stock"') label(2 `"`modelfile1' a3 stock"'))
		graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_5a2.png, width(2000) replace
		
		line `modelfile2'_stock stock q if q>=tq(2001q1) & q<=tq(2024q2), by(oca, yrescale title(`"Comparison - total stock, by oca"')) legend(label(1 `"`modelfile2' total stock"') label(2 `"`modelfile1' total stock"'))
		graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_6oca.png, width(2000) replace
		
		line `modelfile2'_stock_rp stock_rp q if q>=tq(2001q1) & q<=tq(2024q2), by(oca, yrescale title(`"Comparison - low sec stock, by oca"')) legend(label(1 `"`modelfile2' ls stock"') label(2 `"`modelfile1' ls stock"'))
		graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_7rppoca.png, width(2000) replace
		
		line `modelfile2'_stock_a1 stock_a1 q if q>=tq(2001q1) & q<=tq(2024q2), by(oca, yrescale title(`"Comparison - parole violators stock, by oca"')) legend(label(1 `"`modelfile2' a1 stock"') label(2 `"`modelfile1' a1 stock"'))
		graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_8a1oca.png, width(2000) replace

		line `modelfile2'_stock_a2 stock_a2 q if q>=tq(2001q1) & q<=tq(2024q2), by(oca, yrescale title(`"Comparison - com. services stock, by oca"')) legend(label(1 `"`modelfile2' a2 stock"') label(2 `"`modelfile1' a2 stock"'))	
		graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_9a2oca.png, width(2000) replace
		
		line `modelfile2'_stock_a3 stock_a3 q if q>=tq(2001q1) & q<=tq(2024q2), by(oca, yrescale title(`"Comparison - prison parole stock, by oca"')) legend(label(1 `"`modelfile2' a3 stock"') label(2 `"`modelfile1' a3 stock"'))	
		graph export simulations_$pcent/`modelfile1'/comp/`modelfile1'_`modelfile2'_10a3oca.png, width(2000) replace

end			

